/**
 * Copyright (c) 2020 Melown Technologies SE
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * *  Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef RENDERDRAWS_HPP_sdfgh46r5h4
#define RENDERDRAWS_HPP_sdfgh46r5h4

#include <memory>

#include "foundation.hpp"

#include "vts-browser/celestial.hpp"
#include "vts-browser/cameraDraws.hpp"

namespace vts
{

class Map;
class Camera;

namespace renderer
{

class Texture;

class VTSR_API RenderDraws : private Immovable
{
public:
    RenderDraws();
    explicit RenderDraws(Camera *cam);

    void swap(Camera *cam);

    CameraDraws draws;
    MapCelestialBody body;
    std::shared_ptr<Texture> atmosphereDensityTexture;
    double elapsedTime;
    bool projected;
    bool lodBlendingWithDithering;

private:
    Map *map;

    friend class RenderView;
};

} // namespace renderer
} // namespace vts

#endif
